// EditDoc.h
// (c) 2003 exeal

#ifndef _EDIT_DOC_H_
#define _EDIT_DOC_H_

#include <string>
#include <stack>
#include <map>
#include "..\..\Manah\DocumentView.h"
#include "..\..\Manah\File.h"


namespace Ascension {

class CEditDoc;
class CEditController;

// CCharPos class definition and implementation
/////////////////////////////////////////////////////////////////////////////

///	sƕʒui[
class CCharPos {
	// f[^o
public:
	unsigned long	m_iLine;	/// s
	unsigned long	m_iChar;	/// ʒu

	// RXgN^
public:
	explicit CCharPos(unsigned long iLine = 0,
			unsigned long iChar = 0) : m_iLine(iLine), m_iChar(iChar) {
	}

	// Zq
public:
	bool operator ==(const CCharPos& pos) const {
		return (m_iLine == pos.m_iLine && m_iChar == pos.m_iChar);
	}
	bool operator !=(const CCharPos& pos) const {
		return (m_iLine != pos.m_iLine || m_iChar != pos.m_iChar);
	}
	bool operator <(const CCharPos& pos) const {
		return (m_iLine < pos.m_iLine
				|| (m_iLine == pos.m_iLine && m_iChar < pos.m_iChar));
	}
	bool operator <=(const CCharPos& pos) const {
		return (*this < pos || *this == pos);
	}
	bool operator >(const CCharPos& pos) const {
		return (m_iLine > pos.m_iLine
				|| (m_iLine == pos.m_iLine && m_iChar > pos.m_iChar));
	}
	bool operator >=(const CCharPos& pos) const {
		return (*this > pos || *this == pos);
	}
};


// COperation interface definition
/////////////////////////////////////////////////////////////////////////////

class COperationUnit;
// ҏW̎
enum EditOperationType {
	EOT_NONE,		// 
	EOT_TYPING,		// ^CsO
	EOT_DELETE,		// 폜
	EOT_REPLACE,	// ㏑
	EOT_PASTE,		// \t
};

/**
 *	炩̕ҏW
 *	@see	CDeleteOperation, CInsertOperation
 */
interface IOperation {
	///	fXgN^
	virtual ~IOperation() {}
	///  <var>pOperationUnit</var> Ɍ
	virtual bool	Concat(COperationUnit* pOperationUnit) = 0;
	/// s
	virtual void	Execute(CEditDoc* pDoc) = 0;
};


// CInsertOperation class definition
/////////////////////////////////////////////////////////////////////////////

class CDeleteOperation;
class COperationUnit;

///	1̑}
class CInsertOperation : public IOperation {

	friend class CDeleteOperation;
	friend class COperationUnit;
	// f[^o
private:
	bool			m_bType;	// ̎ (true: }Afalse: 폜)
	CCharPos		m_pos;		// }ʒu
	std::wstring	m_strText;	// }

	// RXgN^
public:
	CInsertOperation(const CCharPos& pos, const std::wstring& strText);

	// \bh
public:
	virtual bool	Concat(COperationUnit* pOperationUnit);
	virtual void	Execute(CEditDoc* pDoc);
};


// CInsertOperation class definition
/////////////////////////////////////////////////////////////////////////////

///	1̍폜
class CDeleteOperation : public IOperation {

	friend class CInsertOperation;
	friend class COperationUnit;

	// f[^o
private:
	bool		m_bType;	// ̎ (true: }Afalse: 폜)
	CCharPos	m_posBegin;	// 폜Jn_
	CCharPos	m_posEnd;	// 폜I_

	// RXgN^
public:
	CDeleteOperation(const CCharPos& posBegin, const CCharPos& posEnd);

	// \bh
public:
	virtual bool	Concat(COperationUnit* pOperationUnit);
	virtual void	Execute(CEditDoc* pDoc);
};


// COpearionUnit class definition
/////////////////////////////////////////////////////////////////////////////

///	܂Ƃ߂ăAhD/hD\ȈȂ
class COperationUnit {
	// f[^o
private:
	bool					m_bPreventConcat;	// 㑱̑̌ȂƂ
	std::stack<IOperation*>	m_stkOpes;			// ̒PʂɊ܂܂Ȃ

	// RXgN^
public:
	COperationUnit(bool bPreventConcat);
	virtual ~COperationUnit();

	// \bh
public:
	bool		DoesPermitConcat() const;
	void		Execute(CEditDoc* pDoc);
	IOperation*	Pop();
	void		Push(IOperation* pOperation);
	IOperation*	Top() const;
};


// CEditDoc class definition
/////////////////////////////////////////////////////////////////////////////

// R[hy[WXgɌȂR[h
/// ʁA݂̕R[h
const UINT	EXTCP_AUTO		= 65500;
/// UTF-5
const UINT	EXTCP_UTF5		= 65506;

// 
/// UTF-16 gGfBA
const UINT	EXTCP_UTF16LE	= 1200;
/// UTF-16 rbOGfBA
const UINT	EXTCP_UTF16BE	= 1201;
/// UTF-32 gGfBA
const UINT	EXTCP_UTF32LE	= 12000;
/// UTF-32 rbOGfBA
const UINT	EXTCP_UTF32BE	= 12001;
/// EUC-JP
const UINT	EXTCP_EUCJP		= 51932;

/// sR[h
enum BreakType {
	/// ϊAw薳Ȃ
	BT_AUTO,
	/// sBUnix W (Lf, U+000A)
	BT_LF,
	/// BMacintosh W (Cr, U+000D)
	BT_CR,
	/// +sBWindows W (CrLf, U+000D U+000A)
	BT_CRLF,
	///	Vs (U+0085)
	BT_NEL,
	/// s؂ (U+2028)
	BT_LS,
	/// i؂ (U+2029)
	BT_PS
};

///	t@C̃I[v[h
enum FileOpenMode {
	///	bNȂ
	FOM_DENYNONE,
	///	̏ݖړĨt@CI[v
	FOM_DENYWRITE,
	///	̓ǂݍݖړĨt@CI[v
	FOM_DENYREAD,
	///	ǂݎpƂĊJ (bN͖)
	FOM_ASREADONLY
};

/// LoadDocument ASaveDocument ŕԂl
enum StreamStatus {
	/// t@C͐Ƀ[h/Z[uꂽ
	SS_OK						= 0x00,
	/// t@C̓ǂݎ/݂ۂꂽ
	SS_ACCESSDENIED				= 0x01,
	/// t@C̃bNۂꂽ
	SS_LOCKDENIED				= 0x02,
	/// w肳ꂽR[hy[WȂ
	SS_ILLEGALCODEPAGE			= 0x03,
	///	s̃G[
	SS_UNKNOWNERROR				= 0x04,	
	/// t@Cւ̃ANZXĂ߂ɁAǂݎp[hŊJ
	SS_READ_READONLY			= 0x11,
	/// t@C̕R[h𔻕ʂłȂ
	SS_READ_UNKNOWNCHARCODE		= 0x12,
	/// t@Cɕsȕ܂܂ĂߊSɓǂݍ߂Ȃ
	SS_READ_ILLEGALCHAR			= 0x13,
	/// ǂݍރt@C傫
	SS_READ_HUGEFILE			= 0x14,
	/// w肳ꂽt@C݂Ȃ
	SS_READ_NOEXISTS			= 0x15,
	///	NłȂ
	SS_READ_CANNOTRESOLVELINK	= 0x16,
	/// L}̂ɏނ߂̗̈悪
	SS_WRITE_FULLDISK			= 0x21,
	/// ǂݎp[hA܂͓ǂݎp̃t@CɏƂ
	SS_WRITE_READONLY			= 0x22,
	///	ϊR[hy[WɑΉ镶
	SS_WRITE_CONTAININVALIDCHAR	= 0x23,
};

/// CopyCurrentFile ADeleteCurrentFile AMoveCurrentFile ŕԂl
enum OperationStatus {
	/// t@C͐ɏI
	OPS_OK,
	/// [U𒆒f (ݔֈړ ̂ݗL)
	OPS_ABORTED,
	/// ǂݎp[hŊJĂ̂ŏłȂ
	OPS_MODEISREADONLY,
	/// Rs[Aړɓ̃t@C݂,
	OPS_ALREADYEXISTS,
	/// t@CJƂɓǂݎpŊJ
	OPS_OPENEDASREADONLY,
	/// t@CĴɎs
	OPS_CANNOTOPEN,
	/// \bhsɃt@CJĂȂ
	OPS_HASNOINSTANCE,
	/// m̃G[ (G[bZ[W GetLastError 擾\)
	OPS_UNKNOWNERROR,
};

/// AhDobt@ɒǉƂ̃tO
enum OperationConcatenationFlag {
	/// ̑삪Ȏƌ̂
	OCF_CONCATFORWARD			= 0x00,
	/// ̑삪Ȏƌ̂֎~
	OCF_PREVENTCONCATFORWARD	= 0x01,
	/// ̑삪̑ƌ̂
	OCF_CONCATBACKWARD			= 0x00,
	/// ̑삪̑ƌ̂֎~
	OCF_PREVENTCONCATBACKWARD	= 0x10,
	/// ^CsO (CEditDoc::PushUndoBuffer ɂ)
	OCF_TYPING					= 0x20,
	/// z (p)
	OCF_VIRTUAL					= 0x80,
	///	Sċ (= OCF_CONCATFORWARD | OCF_CONCATBACKWARD)
	OCF_CONCATALL				= 0x00,
	///	Sċ֎~ (= OCF_PREVENTCONCATFORWARD | OCF_PREVENTCONCATBACKWARD)
	OCF_PREVENTALL				= 0x11
};

///	CEditDoc::GetModification œXV
struct TUpdateInfo {
	///	}ł trueB폜ł false
	bool		bIsInsert;
	///	ΏۊJnʒu
	CCharPos	posBegin;
	///	ΏۏIʒu (posBegin &lt;= posEnd)
	CCharPos	posEnd;
	///	I̐Lbgʒu
	CCharPos	posResult;
};

///	s̓e
class CEditDocLine {
	friend class CEditDoc;

	// f[^o
private:
	std::wstring	m_strLine;				// s
	BreakType		m_breakType;			// s̎
	unsigned long	m_cOperationHistory;	// AhDJE^ (0ŕύX̏)

	// RXgN^
public:
	explicit CEditDocLine(std::wstring& strLine = wstring(L""), BreakType breakType = BT_AUTO, bool bModified = false)
		: m_strLine(strLine), m_breakType(breakType), m_cOperationHistory(bModified ? 1 : 0) {
	}

	// \bh
public:
	const std::wstring&	GetLine() const {
		return m_strLine;
	}
	BreakType	GetBreakType() const {
		return m_breakType;
	}
	bool	IsModified() const {
		return m_cOperationHistory != 0;
	}
};

///	hLg̒ʒm󂯎Cxgnh
interface IEditDocEventListener {
	///	fXgN^
	virtual ~IEditDocEventListener() {
	}
	///	hLgύXꂽ
	virtual void	OnDocumentModified() = 0;
	/**
	 *	t@COvZXŕύXꂽ
	 *	@param pDocument	hLg
	 */
	virtual void	OnDocumentOverwrittenByOtherProcess(CEditDoc* pDocument) = 0;
};

typedef std::list<CEditDocLine>				CEditDocLineList;
typedef CEditDocLineList::const_iterator	LineIterator;
typedef std::basic_string<unsigned long>	xstring;	// UTF-32 

typedef unsigned short	SaveDocumentOption;
const SaveDocumentOption	SDO_WRITE_BOM			= 0x0001;	///	BOM 
const SaveDocumentOption	SDO_IGNORE_NOFITCHARS	= 0x0002;	///	ΉȂɑ΂Ċ̕ւ

/**
 *	@brief	Ascension eLXgGfB^̃hLg
 *
 *	SĂ̍s̓eA엚AR[hy[WAsR[hȂǂB
 *	t@Cs
 *
 *	@see	CEditView, CEditController
 */
class CEditDoc : public Manah::Windows::CDocument {

	// CUndoManager class definition
	/////////////////////////////////////////////////////////////////////////

	class CUndoManager /*: public Manah::CManahObject*/ {
		// f[^o
	private:
		CEditDoc*					m_pDoc;			// ΏۃhLg
		std::stack<COperationUnit*>	m_stkUndo;		// AhDX^bN
		std::stack<COperationUnit*>	m_stkRedo;		// hDX^bN
		bool						m_bVirtual;		// ẑƂ^
		COperationUnit*				m_pVirtualUnit;	// zǉ鑀P
		COperationUnit*				m_pLastUnit;	// ŌɒǉꂽP

		// RXgN^
	public:
		CUndoManager(CEditDoc* pDoc);
		virtual ~CUndoManager();

		// \bh
	public:
		void	Clear();								// obt@ɂ
		size_t	GetRedoBufferLength() const;			// hD\ȉ񐔂Ԃ
		size_t	GetUndoBufferLength() const;			// AhD\ȉ񐔂Ԃ
		void	PushUndoBuffer(IOperation* pOperation,	// AhDobt@ɑǉ
					OperationConcatenationFlag ocf);
		void	Redo();									// hLgɃhDs
		void	Undo();									// hLgɃAhDs
	};

	// f[^o
private:
	bool						m_bIgnoreViews;	// OnUpdate ĂяoKvƂ true (t@CǂݍݓrȂ)
	bool						m_bReadOnly;	// ǂݎp[h (ǂݍݎɐݒ肳s)
	FileOpenMode				m_fomMode;		// t@C̋L[h (FOM_ASREADONLY ͖Ӗ)
	UINT						m_nCodePage;	// R[hy[W
	BreakType					m_breakType;	// sR[h (L[{[h͂ɂsɎgp)
	CEditDocLineList			m_listLines;	// s
	Manah::Windows::IO::CFile	m_oFile;		// ݊JĂt@C
	CUndoManager*				m_pUndoManager;	// AhD/hD̊Ǘ
	TUpdateInfo					m_oUpdateInfo;	// Ō̍XV
	bool						m_bOnceUndoBufferCleared;	// 1xȏAhDobt@NA

	bool						m_bVirtualOperating;	// AhD/hDɂ DeleteText AInsertText
														// \bhĂяôƂ true B
														// true ̊Ԃ͗\bhŃhDX^bNNAȂ

	IEditDocEventListener*		m_pEventListener;	// CxgXi
	FILETIME					m_lastWriteTime;	// t@C̍ŏIXV
	UINT						m_nTimerId;			// ^C} ID
	static std::map<UINT, CEditDoc*>	m_mapDocuments;	// ^C} ID -> hLg̃}bv

	mutable unsigned long						m_iLineCache_;	// GetInternalLineIterator LbVpf[^
	mutable CEditDocLineList::iterator			m_itCache_;
	mutable unsigned long						m_iLineCache;	// GetLineIterator LbVpf[^
	mutable CEditDocLineList::const_iterator	m_itCache;
public:
	static wchar_t*		m_arrBreakStrings[7];	// BreakType -> s̃}bv
	static wchar_t		m_wszBreakChars[6];		// s̏W (m_arrBreakStrings ƕ킵)

	// RXgN^
public:
	CEditDoc(CEditController* pController = 0);
	virtual ~CEditDoc();

	// \bh
public:
	// hLgANZX
	void				ClearUndoBuffer();												// AhD/hDX^bÑNA
	void				GetAllLines(std::wstring& strText) const;						// Ss擾
	CEditController*	GetController() const;											// Rg[Ԃ
	unsigned long		GetDocumentLength() const;										// hLg̒Ԃ
	const std::wstring&	GetLine(unsigned long iLine) const throw(out_of_range);			// 1s擾
	unsigned long		GetLineCount() const;											// s擾
	unsigned long		GetLineIndex(unsigned long iLine,								// s̃ItZbg擾
							bool bIncludeBreak) const throw(out_of_range);
	const CEditDocLine*	GetLineInfo(unsigned long iLine) const throw(out_of_range);		// s擾
	LineIterator		GetLineIterator(unsigned long iLine) const throw(out_of_range);	// sCe[^̎擾
	unsigned long		GetLineLength(unsigned long iLine) const throw(out_of_range);	// s̒擾
	void				GetModification(TUpdateInfo& oUpdateInfo) const;				// XVe̎擾
	FileOpenMode		GetShareMode() const;											// r[h̎擾
	unsigned long		GetUndoHistoryLength(bool bRedo = false) const;					// AhD/hD\ȉ񐔂Ԃ
	void				InitiateDocument();												// 
	bool				IsReadOnly() const;												// ǂݎp[h̎擾
	void				SetReadOnly(bool bReadOnly = true);								// ǂݎp[h̐ݒ

	// ҏW
	CCharPos	DeleteText(Manah::Windows::CView* pSender, const CCharPos& posBegin,	// w͈͂폜
					const CCharPos& posEnd, OperationConcatenationFlag ocf);
	CCharPos	InsertText(Manah::Windows::CView* pSender, const CCharPos& pos,			// wʒuɕ}
					const std::wstring& strText, OperationConcatenationFlag ocf);	//  I_̈ʒuԂ
	void		Redo();														// hD̎s
	void		SetEventListener(IEditDocEventListener* pEventListener);	// CxgXi̐ݒ
	void		Undo();														// AhD̎s

	// R[h
	BreakType	GetBreakType() const;									// ݂̉sR[hԂ
	UINT		GetCodePage() const;									// ݂̃R[hy[WԂ
	static bool	IsAvailableCodePage(UINT nCodePage);					// ̃NXŗp\ȃR[hy[Wǂ
	bool		IsCharCodeUnicode() const;								// gp̕R[h Unicode 
	void		SetBreakType(BreakType bt) throw(invalid_argument);		// sR[h̐ݒ
	void		SetCodePage(UINT nCodePage) throw(invalid_argument);	// R[hy[W̐ݒ

	// t@C <-> hLg
	// 3̃\bh͉炩̗RŎsꍇ́AG[̓e\_CAO\A
	// false ܂ StreamStatus ԂB͂̃\bhŐZ
	bool			CloseDocument();										// t@C
	StreamStatus	LoadDocument(const std::wstring& strPathName,			// t@C̓ǂݍ
						FileOpenMode fom, UINT nCodePage = EXTCP_AUTO);
	StreamStatus	SaveDocument(const std::wstring& strPathName,			// t@C̕ۑ
						SaveDocumentOption sdo,
						BreakType bt, UINT nCodePage = EXTCP_AUTO);

	// ɊJĂt@Cɑ΂鏈
	OperationStatus	CopyCurrentFile(const wchar_t* pwszDestination);				// Rs[
	OperationStatus	DeleteCurrentFile();											// ݔֈړ
	bool			LockCurrentFile(FileOpenMode fom);								// bN
	OperationStatus	MoveCurrentFile(const wchar_t* pwszDestination);				// ړ
	bool			SendCurrentFile(bool bAsAttachment, bool bShowDialog = true);	// [֑M
protected:
	CEditDocLineList::iterator
		GetInternalLineIterator(unsigned long iLine) const throw(out_of_range);	// const łȂsCe[^̎擾
	void	CheckFileLastWriteTime();
private:
	static void CALLBACK	TimerProc(HWND hWnd, UINT nMsg, UINT_PTR idEvent, DWORD dwTime);

public:
	virtual bool	OnNewDocument();									// VhLg쐬ƂɌĂяo
	virtual bool	OnOpenDocument(const std::wstring& strPathName);	// t@CJƂɌĂяo
	virtual bool	OnSaveDocument(const std::wstring& strPathName);	// t@CۑƂɌĂяo
};


// CEditDoc class inline implementation
/////////////////////////////////////////////////////////////////////////////

///	ݎgpĂsR[h擾
inline BreakType CEditDoc::GetBreakType() const {
	AssertValid();
	return m_breakType;
}

///	ݎgpĂR[hy[W擾
inline UINT CEditDoc::GetCodePage() const {
	AssertValid();
	return m_nCodePage;
}

///	Rg[擾
inline CEditController* CEditDoc::GetController() const {
	AssertValid();
	return reinterpret_cast<CEditController*>(m_pController);
}

///	s擾
inline unsigned long CEditDoc::GetLineCount() const {
	AssertValid();
	return m_listLines.size();
}

///	Ō̍XV擾
inline void CEditDoc::GetModification(TUpdateInfo& oUpdateInfo) const {
	AssertValid();
	oUpdateInfo = m_oUpdateInfo;
}

///	t@C̔r[h擾
inline FileOpenMode CEditDoc::GetShareMode() const {
	AssertValid();
	return m_fomMode;
}

///	AhDAhD\ȉ񐔂擾
inline unsigned long CEditDoc::GetUndoHistoryLength(bool bRedo /* = false */) const {
	AssertValid();
	return bRedo ? m_pUndoManager->GetRedoBufferLength() : m_pUndoManager->GetUndoBufferLength();
}

///	̃NXŗp\ȃR[hy[WǂԂ (EXTCP_AUTO ɂ false Ԃ)
inline bool CEditDoc::IsAvailableCodePage(UINT nCodePage) {
	return nCodePage == CP_UTF8
		|| nCodePage == EXTCP_EUCJP
		|| nCodePage == EXTCP_UTF16LE
		|| nCodePage == EXTCP_UTF16BE
		|| nCodePage == EXTCP_UTF32LE
		|| nCodePage == EXTCP_UTF32BE
		|| nCodePage == EXTCP_UTF5
		|| ::IsValidCodePage(nCodePage);
}

///	ݎgpĂR[hy[W Unicode Ԃ
inline bool CEditDoc::IsCharCodeUnicode() const {
	AssertValid();
	return m_nCodePage == EXTCP_UTF5
		|| m_nCodePage == CP_UTF7
		|| m_nCodePage == CP_UTF8
		|| m_nCodePage == EXTCP_UTF16LE
		|| m_nCodePage == EXTCP_UTF16BE
		|| m_nCodePage == EXTCP_UTF16LE
		|| m_nCodePage == EXTCP_UTF32BE;
}

///	ǂݎp[hǂԂ
inline bool CEditDoc::IsReadOnly() const {
	AssertValid();
	return m_bReadOnly;
}

} // namespace Ascension


#endif /* _EDIT_DOC_H_ */

/* [EOF] */